@@ -5,6 +5,7 @@ import numpy as np
 from numpy.f2py.tests import util
 
 
+@pytest.mark.slow
 class TestCharacterString(util.F2PyTest):
     # options = ['--debug-capi', '--build-dir', '/tmp/test-build-f2py']
     suffix = '.f90'
@@ -101,7 +102,7 @@ class TestCharacterString(util.F2PyTest):
                       {'1': 'A', '3': 'ABC', 'star': 'ABCDE' * 3}[length],
                       ], dtype='S')
 
-        expected = np.array([[c for c in s] for s in a], dtype='u1')
+        expected = np.array([list(s) for s in a], dtype='u1')
         assert_array_equal(f(a), expected)
 
     @pytest.mark.parametrize("length", length_list)
@@ -113,7 +114,7 @@ class TestCharacterString(util.F2PyTest):
             [{'1': 'a', '3': 'abc', 'star': 'abcde' * 3}[length],
              {'1': 'A', '3': 'ABC', 'star': 'ABCDE' * 3}[length]], dtype='S')
 
-        a = np.array([[c for c in s] for s in expected], dtype='u1')
+        a = np.array([list(s) for s in expected], dtype='u1')
         assert_array_equal(f(a), expected)
 
     @pytest.mark.parametrize("length", length_list)
@@ -126,7 +127,7 @@ class TestCharacterString(util.F2PyTest):
                       [{'1': 'f', '3': 'fgh', 'star': 'fghij' * 3}[length],
                        {'1': 'F', '3': 'FGH', 'star': 'FGHIJ' * 3}[length]]],
                      dtype='S')
-        expected = np.array([[[c for c in item] for item in row] for row in a],
+        expected = np.array([[list(item) for item in row] for row in a],
                             dtype='u1', order='F')
         assert_array_equal(f(a), expected)
 
@@ -512,6 +513,7 @@ class TestMiscCharacter(util.F2PyTest):
        end subroutine {fprefix}_character_bc_old
     """)
 
+    @pytest.mark.slow
     def test_gh18684(self):
         # Test character(len=5) and character*5 usages
         f = getattr(self.module, self.fprefix + '_gh18684')
@@ -574,7 +576,6 @@ class TestMiscCharacter(util.F2PyTest):
 class TestStringScalarArr(util.F2PyTest):
     sources = [util.getpath("tests", "src", "string", "scalar_string.f90")]
 
-    @pytest.mark.slow
     def test_char(self):
         for out in (self.module.string_test.string,
                     self.module.string_test.string77):
@@ -583,7 +584,6 @@ class TestStringScalarArr(util.F2PyTest):
             expected = '|S8'
             assert out.dtype == expected
 
-    @pytest.mark.slow
     def test_char_arr(self):
         for out in (self.module.string_test.strarr,
                     self.module.string_test.strarr77):
@@ -591,3 +591,49 @@ class TestStringScalarArr(util.F2PyTest):
             assert out.shape == expected
             expected = '|S12'
             assert out.dtype == expected
+
+class TestStringAssumedLength(util.F2PyTest):
+    sources = [util.getpath("tests", "src", "string", "gh24008.f")]
+
+    def test_gh24008(self):
+        self.module.greet("joe", "bob")
+
+@pytest.mark.slow
+class TestStringOptionalInOut(util.F2PyTest):
+    sources = [util.getpath("tests", "src", "string", "gh24662.f90")]
+
+    def test_gh24662(self):
+        self.module.string_inout_optional()
+        a = np.array('hi', dtype='S32')
+        self.module.string_inout_optional(a)
+        assert "output string" in a.tobytes().decode()
+        with pytest.raises(Exception):
+            aa = "Hi"
+            self.module.string_inout_optional(aa)
+
+
+@pytest.mark.slow
+class TestNewCharHandling(util.F2PyTest):
+    # from v1.24 onwards, gh-19388
+    sources = [
+        util.getpath("tests", "src", "string", "gh25286.pyf"),
+        util.getpath("tests", "src", "string", "gh25286.f90")
+    ]
+    module_name = "_char_handling_test"
+
+    def test_gh25286(self):
+        info = self.module.charint('T')
+        assert info == 2
+
+@pytest.mark.slow
+class TestBCCharHandling(util.F2PyTest):
+    # SciPy style, "incorrect" bindings with a hook
+    sources = [
+        util.getpath("tests", "src", "string", "gh25286_bc.pyf"),
+        util.getpath("tests", "src", "string", "gh25286.f90")
+    ]
+    module_name = "_char_handling_test"
+
+    def test_gh25286(self):
+        info = self.module.charint('T')
+        assert info == 2
