@@ -6,16 +6,12 @@ matplotlib.  They have been rewritten and extended for convenience.
 
 """
 import itertools
+
 import numpy as np
 import numpy.ma as ma
-from numpy import ndarray, recarray
-from numpy.ma import MaskedArray
-from numpy.ma.mrecords import MaskedRecords
-from numpy.core.overrides import array_function_dispatch
+import numpy.ma.mrecords as mrec
+from numpy._core.overrides import array_function_dispatch
 from numpy.lib._iotools import _is_string_like
-from numpy.testing import suppress_warnings
-
-_check_fill_value = np.ma.core._check_fill_value
 
 
 __all__ = [
@@ -52,6 +48,7 @@ def recursive_fill_fields(input, output):
 
     Examples
     --------
+    >>> import numpy as np
     >>> from numpy.lib import recfunctions as rfn
     >>> a = np.array([(1, 10.), (2, 20.)], dtype=[('A', np.int64), ('B', np.float64)])
     >>> b = np.zeros((3,), dtype=a.dtype)
@@ -84,6 +81,7 @@ def _get_fieldspec(dtype):
 
     Examples
     --------
+    >>> import numpy as np
     >>> dt = np.dtype([(('a', 'A'), np.int64), ('b', np.double, 3)])
     >>> dt.descr
     [(('a', 'A'), '<i8'), ('b', '<f8', (3,))]
@@ -105,7 +103,8 @@ def _get_fieldspec(dtype):
 
 def get_names(adtype):
     """
-    Returns the field names of the input datatype as a tuple.
+    Returns the field names of the input datatype as a tuple. Input datatype
+    must have fields otherwise error is raised.
 
     Parameters
     ----------
@@ -114,16 +113,12 @@ def get_names(adtype):
 
     Examples
     --------
+    >>> import numpy as np
     >>> from numpy.lib import recfunctions as rfn
-    >>> rfn.get_names(np.empty((1,), dtype=int))
-    Traceback (most recent call last):
-        ...
-    AttributeError: 'numpy.ndarray' object has no attribute 'names'
-
-    >>> rfn.get_names(np.empty((1,), dtype=[('A',int), ('B', float)]))
-    Traceback (most recent call last):
-        ...
-    AttributeError: 'numpy.ndarray' object has no attribute 'names'
+    >>> rfn.get_names(np.empty((1,), dtype=[('A', int)]).dtype)
+    ('A',)
+    >>> rfn.get_names(np.empty((1,), dtype=[('A',int), ('B', float)]).dtype)
+    ('A', 'B')
     >>> adtype = np.dtype([('a', int), ('b', [('ba', int), ('bb', int)])])
     >>> rfn.get_names(adtype)
     ('a', ('b', ('ba', 'bb')))
@@ -141,8 +136,9 @@ def get_names(adtype):
 
 def get_names_flat(adtype):
     """
-    Returns the field names of the input datatype as a tuple. Nested structure
-    are flattened beforehand.
+    Returns the field names of the input datatype as a tuple. Input datatype
+    must have fields otherwise error is raised.
+    Nested structure are flattened beforehand.
 
     Parameters
     ----------
@@ -151,15 +147,12 @@ def get_names_flat(adtype):
 
     Examples
     --------
+    >>> import numpy as np
     >>> from numpy.lib import recfunctions as rfn
-    >>> rfn.get_names_flat(np.empty((1,), dtype=int)) is None
-    Traceback (most recent call last):
-        ...
-    AttributeError: 'numpy.ndarray' object has no attribute 'names'
-    >>> rfn.get_names_flat(np.empty((1,), dtype=[('A',int), ('B', float)]))
-    Traceback (most recent call last):
-        ...
-    AttributeError: 'numpy.ndarray' object has no attribute 'names'
+    >>> rfn.get_names_flat(np.empty((1,), dtype=[('A', int)]).dtype) is None
+    False
+    >>> rfn.get_names_flat(np.empty((1,), dtype=[('A',int), ('B', str)]).dtype)
+    ('A', 'B')
     >>> adtype = np.dtype([('a', int), ('b', [('ba', int), ('bb', int)])])
     >>> rfn.get_names_flat(adtype)
     ('a', 'b', 'ba', 'bb')
@@ -180,6 +173,7 @@ def flatten_descr(ndtype):
 
     Examples
     --------
+    >>> import numpy as np
     >>> from numpy.lib import recfunctions as rfn
     >>> ndtype = np.dtype([('a', '<i4'), ('b', [('ba', '<f8'), ('bb', '<i4')])])
     >>> rfn.flatten_descr(ndtype)
@@ -243,10 +237,11 @@ def get_fieldstructure(adtype, lastname=None, parents=None,):
     lastname : optional
         Last processed field name (used internally during recursion).
     parents : dictionary
-        Dictionary of parent fields (used interbally during recursion).
+        Dictionary of parent fields (used internally during recursion).
 
     Examples
     --------
+    >>> import numpy as np
     >>> from numpy.lib import recfunctions as rfn
     >>> ndtype =  np.dtype([('A', int),
     ...                     ('B', [('BA', int),
@@ -268,7 +263,7 @@ def get_fieldstructure(adtype, lastname=None, parents=None,):
                 parents[name] = []
             parents.update(get_fieldstructure(current, name, parents))
         else:
-            lastparent = [_ for _ in (parents.get(lastname, []) or [])]
+            lastparent = list((parents.get(lastname, []) or []))
             if lastparent:
                 lastparent.append(lastname)
             elif lastname:
@@ -335,15 +330,15 @@ def _fix_output(output, usemask=True, asrecarray=False):
     Private function: return a recarray, a ndarray, a MaskedArray
     or a MaskedRecords depending on the input parameters
     """
-    if not isinstance(output, MaskedArray):
+    if not isinstance(output, ma.MaskedArray):
         usemask = False
     if usemask:
         if asrecarray:
-            output = output.view(MaskedRecords)
+            output = output.view(mrec.MaskedRecords)
     else:
         output = ma.filled(output)
         if asrecarray:
-            output = output.view(recarray)
+            output = output.view(np.recarray)
     return output
 
 
@@ -387,6 +382,7 @@ def merge_arrays(seqarrays, fill_value=-1, flatten=False,
 
     Examples
     --------
+    >>> import numpy as np
     >>> from numpy.lib import recfunctions as rfn
     >>> rfn.merge_arrays((np.array([1, 2]), np.array([10., 20., 30.])))
     array([( 1, 10.), ( 2, 20.), (-1, 30.)],
@@ -418,7 +414,7 @@ def merge_arrays(seqarrays, fill_value=-1, flatten=False,
     if (len(seqarrays) == 1):
         seqarrays = np.asanyarray(seqarrays[0])
     # Do we have a single ndarray as input ?
-    if isinstance(seqarrays, (ndarray, np.void)):
+    if isinstance(seqarrays, (np.ndarray, np.void)):
         seqdtype = seqarrays.dtype
         # Make sure we have named fields
         if seqdtype.names is None:
@@ -429,13 +425,13 @@ def merge_arrays(seqarrays, fill_value=-1, flatten=False,
             # Find what type of array we must return
             if usemask:
                 if asrecarray:
-                    seqtype = MaskedRecords
+                    seqtype = mrec.MaskedRecords
                 else:
-                    seqtype = MaskedArray
+                    seqtype = ma.MaskedArray
             elif asrecarray:
-                seqtype = recarray
+                seqtype = np.recarray
             else:
-                seqtype = ndarray
+                seqtype = np.ndarray
             return seqarrays.view(dtype=seqdtype, type=seqtype)
         else:
             seqarrays = (seqarrays,)
@@ -459,8 +455,8 @@ def merge_arrays(seqarrays, fill_value=-1, flatten=False,
             mask = ma.getmaskarray(a).ravel()
             # Get the filling value (if needed)
             if nbmissing:
-                fval = _check_fill_value(fill_value, a.dtype)
-                if isinstance(fval, (ndarray, np.void)):
+                fval = mrec._check_fill_value(fill_value, a.dtype)
+                if isinstance(fval, (np.ndarray, np.void)):
                     if len(fval.dtype) == 1:
                         fval = fval.item()[0]
                         fmsk = True
@@ -478,15 +474,15 @@ def merge_arrays(seqarrays, fill_value=-1, flatten=False,
         output = ma.array(np.fromiter(data, dtype=newdtype, count=maxlength),
                           mask=list(_izip_records(seqmask, flatten=flatten)))
         if asrecarray:
-            output = output.view(MaskedRecords)
+            output = output.view(mrec.MaskedRecords)
     else:
         # Same as before, without the mask we don't need...
         for (a, n) in zip(seqarrays, sizes):
             nbmissing = (maxlength - n)
             data = a.ravel().__array__()
             if nbmissing:
-                fval = _check_fill_value(fill_value, a.dtype)
-                if isinstance(fval, (ndarray, np.void)):
+                fval = mrec._check_fill_value(fill_value, a.dtype)
+                if isinstance(fval, (np.ndarray, np.void)):
                     if len(fval.dtype) == 1:
                         fval = fval.item()[0]
                     else:
@@ -497,7 +493,7 @@ def merge_arrays(seqarrays, fill_value=-1, flatten=False,
         output = np.fromiter(tuple(_izip_records(seqdata, flatten=flatten)),
                              dtype=newdtype, count=maxlength)
         if asrecarray:
-            output = output.view(recarray)
+            output = output.view(np.recarray)
     # And we're done...
     return output
 
@@ -513,10 +509,6 @@ def drop_fields(base, drop_names, usemask=True, asrecarray=False):
 
     Nested fields are supported.
 
-    ..versionchanged: 1.18.0
-        `drop_fields` returns an array with 0 fields if all fields are dropped,
-        rather than returning ``None`` as it did previously.
-
     Parameters
     ----------
     base : array
@@ -533,6 +525,7 @@ def drop_fields(base, drop_names, usemask=True, asrecarray=False):
 
     Examples
     --------
+    >>> import numpy as np
     >>> from numpy.lib import recfunctions as rfn
     >>> a = np.array([(1, (2, 3.0)), (4, (5, 6.0))],
     ...   dtype=[('a', np.int64), ('b', [('ba', np.double), ('bb', np.int64)])])
@@ -628,6 +621,7 @@ def rename_fields(base, namemapper):
 
     Examples
     --------
+    >>> import numpy as np
     >>> from numpy.lib import recfunctions as rfn
     >>> a = np.array([(1, (2, [3.0, 30.])), (4, (5, [6.0, 60.]))],
     ...   dtype=[('a', int),('b', [('ba', float), ('bb', (float, 2))])])
@@ -699,7 +693,7 @@ def append_fields(base, names, data, dtypes=None,
         data = [data, ]
     #
     if dtypes is None:
-        data = [np.array(a, copy=False, subok=True) for a in data]
+        data = [np.array(a, copy=None, subok=True) for a in data]
         data = [a.view([(name, a.dtype)]) for (name, a) in zip(names, data)]
     else:
         if not isinstance(dtypes, (tuple, list)):
@@ -710,7 +704,7 @@ def append_fields(base, names, data, dtypes=None,
             else:
                 msg = "The dtypes argument must be None, a dtype, or a list."
                 raise ValueError(msg)
-        data = [np.array(a, copy=False, subok=True, dtype=d).view([(n, d)])
+        data = [np.array(a, copy=None, subok=True, dtype=d).view([(n, d)])
                 for (a, n, d) in zip(data, names, dtypes)]
     #
     base = merge_arrays(base, usemask=usemask, fill_value=fill_value)
@@ -784,7 +778,8 @@ def repack_fields(a, align=False, recurse=False):
 
     This method removes any overlaps and reorders the fields in memory so they
     have increasing byte offsets, and adds or removes padding bytes depending
-    on the `align` option, which behaves like the `align` option to `np.dtype`.
+    on the `align` option, which behaves like the `align` option to
+    `numpy.dtype`.
 
     If `align=False`, this method produces a "packed" memory layout in which
     each field starts at the byte the previous field ended, and any padding
@@ -811,6 +806,7 @@ def repack_fields(a, align=False, recurse=False):
 
     Examples
     --------
+    >>> import numpy as np
 
     >>> from numpy.lib import recfunctions as rfn
     >>> def print_offsets(d):
@@ -819,7 +815,8 @@ def repack_fields(a, align=False, recurse=False):
     ...
     >>> dt = np.dtype('u1, <i8, <f8', align=True)
     >>> dt
-    dtype({'names':['f0','f1','f2'], 'formats':['u1','<i8','<f8'], 'offsets':[0,8,16], 'itemsize':24}, align=True)
+    dtype({'names': ['f0', 'f1', 'f2'], 'formats': ['u1', '<i8', '<f8'], \
+'offsets': [0, 8, 16], 'itemsize': 24}, align=True)
     >>> print_offsets(dt)
     offsets: [0, 8, 16]
     itemsize: 24
@@ -891,6 +888,50 @@ def _get_fields_and_offsets(dt, offset=0):
                     fields.extend([(d, c, o + i*size) for d, c, o in subfields])
     return fields
 
+def _common_stride(offsets, counts, itemsize):
+    """
+    Returns the stride between the fields, or None if the stride is not
+    constant. The values in "counts" designate the lengths of
+    subarrays. Subarrays are treated as many contiguous fields, with
+    always positive stride.
+    """
+    if len(offsets) <= 1:
+        return itemsize
+
+    negative = offsets[1] < offsets[0]  # negative stride
+    if negative:
+        # reverse, so offsets will be ascending
+        it = zip(reversed(offsets), reversed(counts))
+    else:
+        it = zip(offsets, counts)
+
+    prev_offset = None
+    stride = None
+    for offset, count in it:
+        if count != 1:  # subarray: always c-contiguous
+            if negative:
+                return None  # subarrays can never have a negative stride
+            if stride is None:
+                stride = itemsize
+            if stride != itemsize:
+                return None
+            end_offset = offset + (count - 1) * itemsize
+        else:
+            end_offset = offset
+
+        if prev_offset is not None:
+            new_stride = offset - prev_offset
+            if stride is None:
+                stride = new_stride
+            if stride != new_stride:
+                return None
+
+        prev_offset = end_offset
+
+    if negative:
+        return -stride
+    return stride
+
 
 def _structured_to_unstructured_dispatcher(arr, dtype=None, copy=None,
                                            casting=None):
@@ -899,7 +940,7 @@ def _structured_to_unstructured_dispatcher(arr, dtype=None, copy=None,
 @array_function_dispatch(_structured_to_unstructured_dispatcher)
 def structured_to_unstructured(arr, dtype=None, copy=False, casting='unsafe'):
     """
-    Converts and n-D structured array into an (n+1)-D unstructured array.
+    Converts an n-D structured array into an (n+1)-D unstructured array.
 
     The new array will have a new last dimension equal in size to the
     number of field-elements of the input array. If not supplied, the output
@@ -916,11 +957,18 @@ def structured_to_unstructured(arr, dtype=None, copy=False, casting='unsafe'):
     dtype : dtype, optional
        The dtype of the output unstructured array.
     copy : bool, optional
-        See copy argument to `ndarray.astype`. If true, always return a copy.
-        If false, and `dtype` requirements are satisfied, a view is returned.
+        If true, always return a copy. If false, a view is returned if
+        possible, such as when the `dtype` and strides of the fields are
+        suitable and the array subtype is one of `numpy.ndarray`,
+        `numpy.recarray` or `numpy.memmap`.
+
+        .. versionchanged:: 1.25.0
+            A view can now be returned if the fields are separated by a
+            uniform stride.
+
     casting : {'no', 'equiv', 'safe', 'same_kind', 'unsafe'}, optional
-        See casting argument of `ndarray.astype`. Controls what kind of data
-        casting may occur.
+        See casting argument of `numpy.ndarray.astype`. Controls what kind of
+        data casting may occur.
 
     Returns
     -------
@@ -929,6 +977,7 @@ def structured_to_unstructured(arr, dtype=None, copy=False, casting='unsafe'):
 
     Examples
     --------
+    >>> import numpy as np
 
     >>> from numpy.lib import recfunctions as rfn
     >>> a = np.zeros(4, dtype=[('a', 'i4'), ('b', 'f4,u2'), ('c', 'f4', 2)])
@@ -965,7 +1014,7 @@ def structured_to_unstructured(arr, dtype=None, copy=False, casting='unsafe'):
     if dtype is None:
         out_dtype = np.result_type(*[dt.base for dt in dts])
     else:
-        out_dtype = dtype
+        out_dtype = np.dtype(dtype)
 
     # Use a series of views and casts to convert to an unstructured array:
 
@@ -975,9 +1024,40 @@ def structured_to_unstructured(arr, dtype=None, copy=False, casting='unsafe'):
                                  'formats': dts,
                                  'offsets': offsets,
                                  'itemsize': arr.dtype.itemsize})
-    with suppress_warnings() as sup:  # until 1.16 (gh-12447)
-        sup.filter(FutureWarning, "Numpy has detected")
-        arr = arr.view(flattened_fields)
+    arr = arr.view(flattened_fields)
+
+    # we only allow a few types to be unstructured by manipulating the
+    # strides, because we know it won't work with, for example, np.matrix nor
+    # np.ma.MaskedArray.
+    can_view = type(arr) in (np.ndarray, np.recarray, np.memmap)
+    if (not copy) and can_view and all(dt.base == out_dtype for dt in dts):
+        # all elements have the right dtype already; if they have a common
+        # stride, we can just return a view
+        common_stride = _common_stride(offsets, counts, out_dtype.itemsize)
+        if common_stride is not None:
+            wrap = arr.__array_wrap__
+
+            new_shape = arr.shape + (sum(counts), out_dtype.itemsize)
+            new_strides = arr.strides + (abs(common_stride), 1)
+
+            arr = arr[..., np.newaxis].view(np.uint8)  # view as bytes
+            arr = arr[..., min(offsets):]  # remove the leading unused data
+            arr = np.lib.stride_tricks.as_strided(arr,
+                                                  new_shape,
+                                                  new_strides,
+                                                  subok=True)
+
+            # cast and drop the last dimension again
+            arr = arr.view(out_dtype)[..., 0]
+
+            if common_stride < 0:
+                arr = arr[..., ::-1]  # reverse, if the stride was negative
+            if type(arr) is not type(wrap.__self__):
+                # Some types (e.g. recarray) turn into an ndarray along the
+                # way, so we have to wrap it again in order to match the
+                # behavior with copy=True.
+                arr = wrap(arr)
+            return arr
 
     # next cast to a packed format with all fields converted to new dtype
     packed_fields = np.dtype({'names': names,
@@ -996,7 +1076,7 @@ def _unstructured_to_structured_dispatcher(arr, dtype=None, names=None,
 def unstructured_to_structured(arr, dtype=None, names=None, align=False,
                                copy=False, casting='unsafe'):
     """
-    Converts and n-D unstructured array into an (n-1)-D structured array.
+    Converts an n-D unstructured array into an (n-1)-D structured array.
 
     The last dimension of the input array is converted into a structure, with
     number of field-elements equal to the size of the last dimension of the
@@ -1019,11 +1099,12 @@ def unstructured_to_structured(arr, dtype=None, names=None, align=False,
     align : boolean, optional
        Whether to create an aligned memory layout.
     copy : bool, optional
-        See copy argument to `ndarray.astype`. If true, always return a copy.
-        If false, and `dtype` requirements are satisfied, a view is returned.
+        See copy argument to `numpy.ndarray.astype`. If true, always return a
+        copy. If false, and `dtype` requirements are satisfied, a view is
+        returned.
     casting : {'no', 'equiv', 'safe', 'same_kind', 'unsafe'}, optional
-        See casting argument of `ndarray.astype`. Controls what kind of data
-        casting may occur.
+        See casting argument of `numpy.ndarray.astype`. Controls what kind of
+        data casting may occur.
 
     Returns
     -------
@@ -1032,6 +1113,7 @@ def unstructured_to_structured(arr, dtype=None, names=None, align=False,
 
     Examples
     --------
+    >>> import numpy as np
 
     >>> from numpy.lib import recfunctions as rfn
     >>> dt = np.dtype([('a', 'i4'), ('b', 'f4,u2'), ('c', 'f4', 2)])
@@ -1063,6 +1145,8 @@ def unstructured_to_structured(arr, dtype=None, names=None, align=False,
     else:
         if names is not None:
             raise ValueError("don't supply both dtype and names")
+        # if dtype is the args of np.dtype, construct it
+        dtype = np.dtype(dtype)
         # sanity check of the input dtype
         fields = _get_fields_and_offsets(dtype)
         if len(fields) == 0:
@@ -1104,7 +1188,7 @@ def apply_along_fields(func, arr):
     """
     Apply function 'func' as a reduction across fields of a structured array.
 
-    This is similar to `apply_along_axis`, but treats the fields of a
+    This is similar to `numpy.apply_along_axis`, but treats the fields of a
     structured array as an extra axis. The fields are all first cast to a
     common type following the type-promotion rules from `numpy.result_type`
     applied to the field's dtypes.
@@ -1113,17 +1197,18 @@ def apply_along_fields(func, arr):
     ----------
     func : function
        Function to apply on the "field" dimension. This function must
-       support an `axis` argument, like np.mean, np.sum, etc.
+       support an `axis` argument, like `numpy.mean`, `numpy.sum`, etc.
     arr : ndarray
        Structured array for which to apply func.
 
     Returns
     -------
     out : ndarray
-       Result of the recution operation
+       Result of the reduction operation
 
     Examples
     --------
+    >>> import numpy as np
 
     >>> from numpy.lib import recfunctions as rfn
     >>> b = np.array([(1, 2, 5), (4, 5, 7), (7, 8 ,11), (10, 11, 12)],
@@ -1214,6 +1299,7 @@ def require_fields(array, required_dtype):
 
     Examples
     --------
+    >>> import numpy as np
 
     >>> from numpy.lib import recfunctions as rfn
     >>> a = np.ones(4, dtype=[('a', 'i4'), ('b', 'f8'), ('c', 'u1')])
@@ -1258,6 +1344,7 @@ def stack_arrays(arrays, defaults=None, usemask=True, asrecarray=False,
 
     Examples
     --------
+    >>> import numpy as np
     >>> from numpy.lib import recfunctions as rfn
     >>> x = np.array([1, 2,])
     >>> rfn.stack_arrays(x) is x
@@ -1272,11 +1359,11 @@ def stack_arrays(arrays, defaults=None, usemask=True, asrecarray=False,
                  mask=[(False, False,  True), (False, False,  True),
                        (False, False, False), (False, False, False),
                        (False, False, False)],
-           fill_value=(b'N/A', 1.e+20, 1.e+20),
+           fill_value=(b'N/A', 1e+20, 1e+20),
                 dtype=[('A', 'S3'), ('B', '<f8'), ('C', '<f8')])
 
     """
-    if isinstance(arrays, ndarray):
+    if isinstance(arrays, np.ndarray):
         return arrays
     elif len(arrays) == 1:
         return arrays[0]
@@ -1347,6 +1434,7 @@ def find_duplicates(a, key=None, ignoremask=True, return_index=False):
 
     Examples
     --------
+    >>> import numpy as np
     >>> from numpy.lib import recfunctions as rfn
     >>> ndtype = [('a', int)]
     >>> a = np.ma.array([1, 1, 1, 2, 2, 3, 3],
@@ -1592,3 +1680,6 @@ def rec_join(key, r1, r2, jointype='inner', r1postfix='1', r2postfix='2',
     kwargs = dict(jointype=jointype, r1postfix=r1postfix, r2postfix=r2postfix,
                   defaults=defaults, usemask=False, asrecarray=True)
     return join_by(key, r1, r2, **kwargs)
+
+
+del array_function_dispatch
