@@ -1,9 +1,11 @@
 from __future__ import annotations
 
-from typing import Any
+from typing import Any, TYPE_CHECKING
 
 import numpy as np
-from numpy._typing import ArrayLike, _SupportsArray
+
+if TYPE_CHECKING:
+    from numpy._typing import NDArray, ArrayLike, _SupportsArray
 
 x1: ArrayLike = True
 x2: ArrayLike = 5
@@ -20,22 +22,24 @@ x12: ArrayLike = memoryview(b'foo')
 
 
 class A:
-    def __array__(self, dtype: None | np.dtype[Any] = None) -> np.ndarray:
-        return np.array([1, 2, 3])
+    def __array__(
+        self, dtype: None | np.dtype[Any] = None
+    ) -> NDArray[np.float64]:
+        return np.array([1.0, 2.0, 3.0])
 
 
 x13: ArrayLike = A()
 
-scalar: _SupportsArray = np.int64(1)
+scalar: _SupportsArray[np.dtype[np.int64]] = np.int64(1)
 scalar.__array__()
-array: _SupportsArray = np.array(1)
+array: _SupportsArray[np.dtype[np.int_]] = np.array(1)
 array.__array__()
 
-a: _SupportsArray = A()
+a: _SupportsArray[np.dtype[np.float64]] = A()
 a.__array__()
 a.__array__()
 
 # Escape hatch for when you mean to make something like an object
 # array.
-object_array_scalar: Any = (i for i in range(10))
+object_array_scalar: object = (i for i in range(10))
 np.array(object_array_scalar)
